
#include <errno.h>
#include <sensors/sensors.h>
#include <sensors/chips.h>
#include <sensors/error.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
//#include <sys/param.h>
//#include <sys/types.h>

#ifndef FALSE
	#define FALSE 1
#endif

#ifndef TRUE
	#define TRUE 0
#endif

#define MAX_FILENAME_LEN 128
#define LM75	0
#define LM78    1
#define LM78J   2
#define LM79    3
#define LM80    4
#define SIS5595 5
#define W83781D 6
#define W83782D 7
#define W83783S 8

#define CELCIUS 0
#define FAHRENHEIT 1
#define KELVIN 2

#define DEFAULT_CONFIG_FILE_NAME "sensors.conf"
static char *config_file_name;
FILE *config_file;
static const char *config_file_path[] =
{ "/etc", "/usr/lib/sensors", "/usr/local/lib/sensors", "/usr/lib",
  "/usr/local/lib", ".", 0 };
/************************/
/* GetLimits() function */
/************************/


/* This examines global var config_file, and leaves the name there too.
   It also opens config_file. */
int open_this_config_file(char *filename)
{
  config_file = fopen(filename,"r");
  if (! config_file)
    return -errno;
  return 0;
}

void open_config_file(void)
{
//#define MAX_FILENAME_LEN 1024
  char *filename;
  char buffer[MAX_FILENAME_LEN];
  int res,i;

  if (config_file_name && !strcmp(config_file_name,"-")) {
    config_file = stdin;
    return;
  } else if (config_file_name && index(config_file_name,'/')) {
    if ((res = open_this_config_file(config_file_name))) {
      fprintf(stderr,"Could not locate or open config file\n");
      fprintf(stderr,"%s: %s\n",config_file_name,strerror(res));
      exit(1);
    }
  }
  else {
    if (config_file_name)
      filename = config_file_name;
    else
      filename = strdup(DEFAULT_CONFIG_FILE_NAME);
    for (i = 0; config_file_path[i]; i++) {
      if ((snprintf(buffer,MAX_FILENAME_LEN,
		    "%s/%s",config_file_path[i],filename)) < 1) {
        fprintf(stderr,
                "open_config_file: ridiculous long config file name!\n");
        exit(1);
      }
      if (!open_this_config_file(buffer)) {
        free(config_file_name);
        config_file_name = strdup(buffer);
        return;
      }
    }
    fprintf(stderr,"Could not locate or open config file!\n");
    exit(1);
  }
}


void GetLimits(double *temp_ll, double *temp2_ul, double *temp3_ul, double *temp1_ul, double *in0_ll, double *in0_ul, double *in1_ll, double *in1_ul, double *in2_ll, double *in2_ul, double *in3_ll, double *in3_ul, double *in4_ll, double *in4_ul, double *in5_ll, double *in5_ul, double *in6_ll, double *in6_ul)
{

  int errors, fd, len, lm75, lm7x, gl518, chip_nr;
  char *p;
  const sensors_chip_name *name;

  /* We set the default limits; these will be used if reading fails. */
  *temp_ll = 20;
  *temp2_ul = 60;
  *temp3_ul = 60;
  *temp1_ul = 60;
  *in0_ll = 1.8;
  *in0_ul = 2.2;
  *in1_ll = 1.8;
  *in1_ul = 2.2;
  *in2_ll = 3.0;
  *in2_ul = 3.6;
  *in3_ll = 4.5;
  *in3_ul = 5.5;
  *in4_ll = 10.80;
  *in4_ul = 13.20;
  *in5_ll = -13.20;
  *in5_ul = -10.80;
  *in6_ll = -5.5;
  *in6_ul = -4.5;

  errors = 0;
  lm75 = 0;
  lm7x = 0;
  gl518 = 0;

  for (chip_nr = 0; (name = sensors_get_detected_chips(&chip_nr));)
  {
    if (!strcmp(name->prefix,"lm78") || !strcmp(name->prefix,"lm78-j") ||
        !strcmp(name->prefix,"lm79") || !strcmp(name->prefix,"sis5595") ||
	!strcmp(name->prefix,"lm80") || !strcmp(name->prefix,"w83781d") ||
	!strcmp(name->prefix,"w83782d") || !strcmp(name->prefix,"w83783s"))

      /* Lots of chips are dealt with here as the same numbers correspond
	 to the same features across different chips. This is nice! */

      if (!lm7x)
      {
	lm7x++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN0_MIN, in0_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN1_MIN, in1_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN2_MIN, in2_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN3_MIN, in3_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN4_MIN, in4_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN5_MIN, in5_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN6_MIN, in6_ul))
	  if (strcmp(name->prefix,"adm9240"))
	    errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN0_MAX, in0_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN1_MAX, in1_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN2_MAX, in2_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN3_MAX, in3_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN4_MAX, in4_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN5_MAX, in5_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN6_MAX, in6_ll))
	  if (strcmp(name->prefix,"adm9240"))
	    errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_TEMP_OVER, temp1_ul))
	  errors++;
        if (!strcmp(name->prefix,"w83781d")) {
	  if (sensors_get_feature(*name,SENSORS_W83781D_TEMP2_OVER, temp2_ul))
	    errors++;
	  if (sensors_get_feature(*name,SENSORS_W83781D_TEMP3_OVER, temp3_ul))
	    errors++;
	}
      }
    if (!strcmp(name->prefix,"gl518sm-r00") ||
	!strcmp(name->prefix,"gl518sm-r80"))
      if (!gl518)
      {
	gl518++;
	if (sensors_get_feature(*name, SENSORS_GL518R00_VDD_MIN, in0_ll))
	  errors++;
        if (sensors_get_feature(*name, SENSORS_GL518R00_VIN1_MIN, in1_ll))
	  errors++;
        if (sensors_get_feature(*name, SENSORS_GL518R00_VIN2_MIN, in2_ll))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_VIN3_MIN, in3_ll))
	  errors++;
	if (sensors_get_feature(*name, SENSORS_GL518R00_VDD_MAX, in0_ul))
	  errors++;
        if (sensors_get_feature(*name, SENSORS_GL518R00_VIN1_MAX, in1_ul))
	  errors++;
        if (sensors_get_feature(*name, SENSORS_GL518R00_VIN2_MAX, in2_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_VIN3_MAX, in3_ul))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_TEMP_OVER, temp1_ul))
	  errors++;
      }
      if (!strcmp(name->prefix,"lm75"))
      {
        if (!lm75)
        {
	  lm75++;
	  if (sensors_get_feature(*name,SENSORS_LM75_TEMP_OVER,temp2_ul))
	    errors++;
        }
        else if (lm75==1)
          if (sensors_get_feature(*name,SENSORS_LM75_TEMP_OVER,temp3_ul))
	    errors++;
      }
  }
}


/********************/
/* GetLm() function */
/********************/

void GetLm(double *temp2, double *temp3, double *temp1, double *in0, double *in1, double *in2, double *in3, double *in6, double *in4, double *in5, double *fan1, double *fan2, double *fan3, int multiple_lm75, double *alrms)
{ 
  char buffer[32];
  int errors, fd, len, lm75, lm7x, gl518, buggy_gl518, chip_nr;
  char *p;
  const sensors_chip_name *name;

  *temp1 = -279;
  *temp2 = -279;
  *temp3 = -279;

  *in0 = -279; 
  *in1 = -279;
  *in2 = -279;
  *in3 = -279;
  *in4 = -279;
  *in5 = -279;
  *in6 = -279;
  *fan1 = 0;
  *fan2 = 0;
  *fan3 = 0;  

  /* Here comes the real code... */
  errors = 0;
  lm75 = 0; /* The variables check we haven't already found one of   */
  lm7x = 0; /* the relevant type of sensor chip and read its values. */
  gl518 = 0;
  buggy_gl518 = 0;

  for (chip_nr = 0; (name = sensors_get_detected_chips(&chip_nr));)
  {
    if (!strcmp(name->prefix,"lm78") || !strcmp(name->prefix,"lm78-j") ||
        !strcmp(name->prefix,"lm79") || !strcmp(name->prefix,"sis5595") ||
	!strcmp(name->prefix,"lm80") || !strcmp(name->prefix,"w83781d"))

      /* Lots of chips are dealt with here as Frodo has used the same
	 numbers to index its features as to index the corresponding
	 LM78 feature. This is nice! */

      if (!lm7x)
      {
	lm7x++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN0, in0))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN1, in1))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN2, in2))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN3, in3))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN4, in4))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN5, in5))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_IN6, in6))
	  if (strcmp(name->prefix,"adm9240"))
	    errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_TEMP, temp1))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_FAN1, fan1))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_FAN2, fan2))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_LM78_FAN3, fan3))
	  if (!(!strcmp(name->prefix,"adm9240") ||
		!strcmp(name->prefix,"lm80")))
	    errors++; /* These chips only have two fans */
	if (sensors_get_feature(*name,SENSORS_LM78_ALARMS, alrms))
	  errors++;
        if (!strcmp(name->prefix,"w83781d")) {
	  if (sensors_get_feature(*name,SENSORS_W83781D_TEMP2, temp2))
	    errors++;
	  if (sensors_get_feature(*name,SENSORS_W83781D_TEMP3, temp3))
	    errors++;
	  if (sensors_get_feature(*name,SENSORS_W83781D_ALARMS, alrms))
	    errors++;
	}
      }
    if (!strcmp(name->prefix,"gl518sm-r00") ||
	!strcmp(name->prefix,"gl518sm-r80"))
      if (!gl518)
      {
	gl518++;
	if (!strcmp(name->prefix,"gl518sm-r00"))
	  buggy_gl518 = 1;
	if (!buggy_gl518 && sensors_get_feature(*name,
						SENSORS_GL518R00_VDD, in0))
	  errors++;
        if (buggy_gl518 && sensors_get_feature(*name,
					       SENSORS_GL518R00_VIN1, in1))
	  errors++;
        if (buggy_gl518 && sensors_get_feature(*name,
					       SENSORS_GL518R00_VIN2, in2))
	  errors++;

	/* The above aren't expected to work for rev 00 chips, but there *
	 * is a klugey hack in existence where the limits are used to    *
         * binary-search for the value; in this case I suppose it might. *
	 * No errors are generated for rev 00 chips.                     */

	/* The rest should work for all gl518s. */
        if (sensors_get_feature(*name,SENSORS_GL518R00_VIN3, in3))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_TEMP, temp1))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_FAN1, fan1))
	  errors++;
        if (sensors_get_feature(*name,SENSORS_GL518R00_FAN2, fan2))
	  errors++;
	if (sensors_get_feature(*name,SENSORS_GL518R00_ALARMS, alrms))
	  errors++;
      }
      if (!strcmp(name->prefix,"lm75"))
      {
        if (!lm75)
        {
	  lm75++;
	  if (sensors_get_feature(*name,SENSORS_LM75_TEMP,temp2))
            errors++;
        }
        else if (lm75==1)
          if (sensors_get_feature(*name,SENSORS_LM75_TEMP,temp3))
            errors++;
      }
  }

  if (errors) {
    sensors_cleanup();
    errors = sensors_init(config_file);
    fprintf(stderr,"sensors_init returned %i\n",errors);
    fprintf(stderr,"Warning: errors occurred reading data. Possibly the sensors interface module\nhas changed while wmsensors is running.\n Attempted to correct this by restarting libsensors.\nFor now, this will cause libsensors to return incorrect values.\n");
  }
}